#include "ggap/script.h"
#include "ggap/worksheet.h"
#include "ggap/wswindow.h"
#include "ggap/app.h"
#include <moo/debug.h>
#include <QtScript>
#include <QDesktopServices>
#include "ggap/script-types.h"

using namespace ggap;


ScriptRunner::ScriptRunner(QObject *parent) :
    QObject(parent)
{
}

ScriptRunner::~ScriptRunner()
{
}

void ScriptRunner::setupEngine(QScriptEngine&, QScriptValue&)
{
}

static void bindMethods(QScriptValue &global_obj, const QScriptValue &self)
{
    QObject *obj = self.toQObject();
    const QMetaObject *meta = obj->metaObject();
    for (int i = 0; i < meta->methodCount(); ++i)
    {
        const QMetaMethod meth = meta->method(i);
        if (meth.access() == QMetaMethod::Public &&
            meth.methodType() == QMetaMethod::Method)
        {
            QString s = meth.signature();
            QString name = s.mid(0, s.indexOf('('));
            global_obj.setProperty(name, self.property(name));
        }
    }
}

void ScriptRunner::runScript(const QString &script)
{
    mDebug("script") << "running script:" << script;

    QScriptEngine engine;

    QScriptValue self = engine.newQObject(this);
    QScriptValue global_obj = engine.globalObject();
    bindMethods(global_obj, self);

    global_obj.setProperty("app", engine.newQObject(gapApp));
    registerMetaTypes(&engine);
    setupEngine(engine, global_obj);

    mDebug("script") << engine.evaluate(script).toString();
}


WorksheetScriptProxy::WorksheetScriptProxy(Worksheet *ws) :
    ScriptRunner(ws),
    ws(ws)
{
}

WorksheetScriptProxy::~WorksheetScriptProxy()
{
}

bool WorksheetScriptProxy::openUrl(const QString &url)
{
    return QDesktopServices::openUrl(url);
}

void WorksheetScriptProxy::setupEngine(QScriptEngine &engine, QScriptValue &global_obj)
{
    global_obj.setProperty("worksheet", engine.newQObject(ws));
}
